



make_sample_chip(samptr)
register Sample *samptr;
{
	ASSERT_SAMPLE_MAGIC(samptr);

	/* chip_use_count is the number of make_sample_chip calls that are
	 * active for the sample.  It must never be less than 0 */
	assert(samptr->chip_use_count >= 0);

	/* if the use count is nonzero, someone else has already locked it
	 * in.  merely increment the count and return.  the assert insures
	 * that we're pointing at data */
	if (samptr->chip_use_count)
	{
		samptr->chip_use_count++;
		assert(samptr->sampledata);
		return(1);
	}

	/* if we allocated it from chip RAM, we don't have to copy it down
	 * there, so we can return 'success' */

	if (samptr->sample_flags & SAMPLE_ALLOCATED_IN_CHIP_RAM)
	{
		assert(samptr->sampledata);
		return(1);
	}

	/* at this point, data should be in fast RAM and not chip RAM, and
	 * the sampledata pointer should be NULL else there's a bug and
	 * we'll be losing memory */
	assert(samptr->sampledata == NULL);

	/* alloc CHIP memory for the sample */
	samptr->sampledata = (char *)AllocMem(samptr->samplebytes,MEMF_CHIP);

	/* if it failed, return failure */
	if (samptr->sampledata == NULL)
	{
		return(0);
	}

	/* copy the sample down, set flags indicated it has been copied
	 * down, and set use count to 1 'cuz we're the first.  
	 *
	 * note, the program can call make_sample_chip multiple times
	 * before a free because the free is performed when the sound is
	 * finished and more than one request could be pending for the
	 * sound, but we don't need a semaphore or anything because the
	 * program isn't multithreaded and no other programs diddle with
	 * this stuff
	 */

	movmem(samptr->fastsampledata,samptr->sampledata,samptr->samplebytes);
	samptr->sample_flags |= SAMPLE_COPIED_INTO_CHIP_RAM;
	samptr->chip_use_count = 1;

#ifdef DEBUG
	printf("copied sample to chip\n");
#endif
	return(1);
}

free_sample_chip(samptr)
register Sample *samptr;
{
	ASSERT_SAMPLE_MAGIC(samptr);

	if (samptr->chip_use_count > 1)
	{
		samptr->chip_use_count--;
		return;
	}

	/* if it was allocated in CHIP RAM, no need to free */
	if (samptr->sample_flags & SAMPLE_ALLOCATED_IN_CHIP_RAM)
	{
		/* if the flag indicates we should free this from the cache
		 * when done playing, do so (for demand-disk-loaded timer
		 * events) */
		if (samptr->sample_flags & PURGE_SAMPLE_AFTER_PLAYING)
			UnloadSampleAt(samptr);

		return;
	}

	/* if we got here, it must be the last free, i.e., the real free,
	 * so insure it (for testing at least) with an assert */
	assert(samptr->chip_use_count == 1);

	/* we are the last user, zero the use count and free the CHIP ram */
	samptr->chip_use_count = 0;

	/* we insist that the sample being freed is currently in CHIP RAM */
	assert(samptr->sample_flags & SAMPLE_COPIED_INTO_CHIP_RAM);

	/* free the CHIP RAM */
	FreeMem(samptr->sampledata,samptr->samplebytes);

	/* clear the indication that it is loaded */
	samptr->sample_flags &= ~(SAMPLE_COPIED_INTO_CHIP_RAM);

	/* null out the data pointer, useful to make asserts trip when there
	 * are bugs and helps keep us from playing garbage by accident */
	samptr->sampledata = NULL;

	/* if the flag indicates we should free this from the cache
	 * when done playing, do so (for demand-disk-loaded timer
	 * events) */
	if (samptr->sample_flags & PURGE_SAMPLE_AFTER_PLAYING)
		UnloadSampleAt(samptr);

#ifdef DEBUG
	printf("freed sample from chip RAM\n");
#endif
}

